@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -54,9 +54,9 @@ class Ray
 {
 public:
     BOOST_STATIC_ASSERT(boost::is_floating_point<RealT>::value);
-    typedef RealT      RealType;
-    typedef Vec3<Real> Vec3Type;
-    typedef Vec3Type   Vec3T;
+    typedef RealT       RealType;
+    typedef Vec3<RealT> Vec3Type;
+    typedef Vec3Type    Vec3T;
     struct TimeSpan {
         RealT t0, t1;
         /// @brief Default constructor
@@ -67,7 +67,7 @@ public:
         inline void set(RealT _t0, RealT _t1) { t0=_t0; t1=_t1; }
         /// @brief Get both times
         inline void get(RealT& _t0, RealT& _t1) const { _t0=t0; _t1=t1; }
-        /// @brief Return @c true if t1 is larger then t0 by at least eps.
+        /// @brief Return @c true if t1 is larger than t0 by at least eps.
         inline bool valid(RealT eps=math::Delta<RealT>::value()) const { return (t1-t0)>eps; }
         /// @brief Return the midpoint of the ray.
         inline RealT mid() const { return 0.5*(t0 + t1); }
@@ -105,7 +105,7 @@ public:
     }
 
     inline void scaleTimes(RealT scale) { mTimeSpan.scale(scale); }
-    
+
     inline void reset(const Vec3Type& eye,
                       const Vec3Type& direction,
                       RealT t0 = math::Delta<RealT>::value(),
@@ -138,21 +138,21 @@ public:
     /// @brief Return the midpoint of the ray.
     inline Vec3R mid() const { return (*this)(mTimeSpan.mid()); }
 
-    /// @brief Return @c true if t0 is strictly less then t1.
+    /// @brief Return @c true if t0 is strictly less than t1.
     OPENVDB_DEPRECATED inline bool test() const { return mTimeSpan.valid(RealT(0)); }
 
-    /// @brief Return @c true if t1 is larger then t0 by at least eps.
+    /// @brief Return @c true if t1 is larger than t0 by at least eps.
     inline bool valid(RealT eps=math::Delta<float>::value()) const
       {
           return mTimeSpan.valid(eps);
       }
-    
+
     /// @brief Return @c true if @a time is within t0 and t1, both inclusive.
     inline bool test(RealT time) const { return mTimeSpan.test(time); }
 
     /// @brief Return a new Ray that is transformed with the specified map.
     /// @param map  the map from which to construct the new Ray.
-    /// @warning Assumes a linear map and a normalize direction.
+    /// @warning Assumes a linear map and a normalized direction.
     /// @details The requirement that the direction is normalized
     /// follows from the transformation of t0 and t1 - and that fact that
     /// we want applyMap and applyInverseMap to be inverse operations.
@@ -160,7 +160,7 @@ public:
     inline Ray applyMap(const MapType& map) const
     {
         assert(map.isLinear());
-        assert(math::isApproxEqual(mDir.length(), RealT(1)));
+        assert(math::isRelOrApproxEqual(mDir.length(), RealT(1), Tolerance<RealT>::value(), Delta<RealT>::value()));
         const Vec3T eye = map.applyMap(mEye);
         const Vec3T dir = map.applyJacobian(mDir);
         const RealT length = dir.length();
@@ -169,7 +169,7 @@ public:
 
     /// @brief Return a new Ray that is transformed with the inverse of the specified map.
     /// @param map  the map from which to construct the new Ray by inverse mapping.
-    /// @warning Assumes a linear map and a normalize direction.
+    /// @warning Assumes a linear map and a normalized direction.
     /// @details The requirement that the direction is normalized
     /// follows from the transformation of t0 and t1 - and that fact that
     /// we want applyMap and applyInverseMap to be inverse operations.
@@ -177,7 +177,7 @@ public:
     inline Ray applyInverseMap(const MapType& map) const
     {
         assert(map.isLinear());
-        assert(math::isApproxEqual(mDir.length(), RealT(1)));
+        assert(math::isRelOrApproxEqual(mDir.length(), RealT(1), Tolerance<RealT>::value(), Delta<RealT>::value()));
         const Vec3T eye = map.applyInverseMap(mEye);
         const Vec3T dir = map.applyInverseJacobian(mDir);
         const RealT length = dir.length();
@@ -193,13 +193,13 @@ public:
     }
 
     /// @brief Return a new ray in the index space of the specified
-    /// grid, assuming the existing ray is represented in world space. 
+    /// grid, assuming the existing ray is represented in world space.
     template<typename GridType>
     inline Ray worldToIndex(const GridType& grid) const
     {
         return this->applyInverseMap(*(grid.transform().baseMap()));
     }
-    
+
     /// @brief Return true if this ray intersects the specified sphere.
     /// @param center The center of the sphere in the same space as this ray.
     /// @param radius The radius of the sphere in the same units as this ray.
@@ -213,7 +213,7 @@ public:
         const RealT A = mDir.lengthSqr();
         const RealT B = 2 * mDir.dot(origin);
         const RealT C = origin.lengthSqr() - radius * radius;
-        const RealT D = B * B - 4 * A * C;   
+        const RealT D = B * B - 4 * A * C;
 
         if (D < 0) return false;
 
@@ -221,13 +221,13 @@ public:
 
         t0 = Q / A;
         t1 = C / Q;
-        
+
         if (t0 > t1) std::swap(t0, t1);
         if (t0 < mTimeSpan.t0) t0 = mTimeSpan.t0;
         if (t1 > mTimeSpan.t1) t1 = mTimeSpan.t1;
         return t0 <= t1;
     }
-    
+
     /// @brief Return true if this ray intersects the specified sphere.
     /// @param center The center of the sphere in the same space as this ray.
     /// @param radius The radius of the sphere in the same units as this ray.
@@ -237,7 +237,7 @@ public:
         return this->intersects(center, radius, t0, t1)>0;
     }
 
-    /// @brief Return true if this ray intersects the specified sphere. 
+    /// @brief Return true if this ray intersects the specified sphere.
     /// @note For intersection this ray is clipped to the two intersection points.
     /// @param center The center of the sphere in the same space as this ray.
     /// @param radius The radius of the sphere in the same units as this ray.
@@ -260,7 +260,7 @@ public:
     inline bool intersects(const BBoxT& bbox, RealT& t0, RealT& t1) const
     {
         mTimeSpan.get(t0, t1);
-        for (size_t i = 0; i < 3; ++i) {
+        for (int i = 0; i < 3; ++i) {
             RealT a = (bbox.min()[i] - mEye[i]) * mInvDir[i];
             RealT b = (bbox.max()[i] - mEye[i]) * mInvDir[i];
             if (a > b) std::swap(a, b);
@@ -280,7 +280,7 @@ public:
         return this->intersects(bbox, t0, t1);
     }
 
-    /// @brief Return true if this ray intersects the specified bounding box. 
+    /// @brief Return true if this ray intersects the specified bounding box.
     /// @note For intersection this ray is clipped to the two intersection points.
     /// @param bbox Axis-aligned bounding box in the same space as this ray.
     template<typename BBoxT>
@@ -319,7 +319,7 @@ private:
     Vec3T mEye, mDir, mInvDir;
     TimeSpan mTimeSpan;
 }; // end of Ray class
-    
+
 /// @brief Output streaming of the Ray class.
 /// @note Primarily intended for debugging.
 template<typename RealT>
@@ -337,6 +337,6 @@ inline std::ostream& operator<<(std::ostream& os, const Ray<RealT>& r)
 
 #endif // OPENVDB_MATH_RAY_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
