@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -28,8 +28,8 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
-#ifndef OPENVDB_INIT_HAS_BEEN_INCLUDED
-#define OPENVDB_INIT_HAS_BEEN_INCLUDED
+#ifndef OPENVDB_OPENVDB_HAS_BEEN_INCLUDED
+#define OPENVDB_OPENVDB_HAS_BEEN_INCLUDED
 
 #include "Platform.h"
 #include "Types.h"
@@ -46,32 +46,33 @@ OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
 
 /// Common tree types
-typedef tree::Tree4<bool,    5, 4, 3>::Type     BoolTree;
-typedef tree::Tree4<float,   5, 4, 3>::Type     FloatTree;
-typedef tree::Tree4<double,  5, 4, 3>::Type     DoubleTree;
-typedef tree::Tree4<int32_t, 5, 4, 3>::Type     Int32Tree;
-typedef tree::Tree4<uint32_t, 5, 4, 3>::Type    UInt32Tree;
-typedef tree::Tree4<int64_t, 5, 4, 3>::Type     Int64Tree;
-typedef tree::Tree4<Hermite, 5, 4, 3>::Type     HermiteTree;
-typedef tree::Tree4<Vec2i, 5, 4, 3>::Type       Vec2ITree;
-typedef tree::Tree4<Vec2s, 5, 4, 3>::Type       Vec2STree;
-typedef tree::Tree4<Vec2d, 5, 4, 3>::Type       Vec2DTree;
-typedef tree::Tree4<Vec3i, 5, 4, 3>::Type       Vec3ITree;
-typedef tree::Tree4<Vec3f, 5, 4, 3>::Type       Vec3STree;
-typedef tree::Tree4<Vec3d, 5, 4, 3>::Type       Vec3DTree;
-typedef tree::Tree4<std::string, 5, 4, 3>::Type StringTree;
+typedef tree::Tree4<ValueMask,   5, 4, 3>::Type  MaskTree;
+typedef tree::Tree4<bool,        5, 4, 3>::Type  BoolTree;
+typedef tree::Tree4<float,       5, 4, 3>::Type  FloatTree;
+typedef tree::Tree4<double,      5, 4, 3>::Type  DoubleTree;
+typedef tree::Tree4<int32_t,     5, 4, 3>::Type  Int32Tree;
+typedef tree::Tree4<uint32_t,    5, 4, 3>::Type  UInt32Tree;
+typedef tree::Tree4<int64_t,     5, 4, 3>::Type  Int64Tree;
+typedef tree::Tree4<Vec2i,       5, 4, 3>::Type  Vec2ITree;
+typedef tree::Tree4<Vec2s,       5, 4, 3>::Type  Vec2STree;
+typedef tree::Tree4<Vec2d,       5, 4, 3>::Type  Vec2DTree;
+typedef tree::Tree4<Vec3i,       5, 4, 3>::Type  Vec3ITree;
+typedef tree::Tree4<Vec3f,       5, 4, 3>::Type  Vec3STree;
+typedef tree::Tree4<Vec3d,       5, 4, 3>::Type  Vec3DTree;
+typedef tree::Tree4<std::string, 5, 4, 3>::Type  StringTree;
+typedef MaskTree  TopologyTree;    
 typedef Vec3STree Vec3fTree;
 typedef Vec3DTree Vec3dTree;
 typedef FloatTree ScalarTree;
 typedef Vec3fTree VectorTree;
 
 /// Common grid types
+typedef Grid<MaskTree>      MaskGrid;
 typedef Grid<BoolTree>      BoolGrid;
 typedef Grid<FloatTree>     FloatGrid;
 typedef Grid<DoubleTree>    DoubleGrid;
 typedef Grid<Int32Tree>     Int32Grid;
 typedef Grid<Int64Tree>     Int64Grid;
-typedef Grid<HermiteTree>   HermiteGrid;
 typedef Grid<Vec3ITree>     Vec3IGrid;
 typedef Grid<Vec3STree>     Vec3SGrid;
 typedef Grid<Vec3DTree>     Vec3DGrid;
@@ -80,7 +81,7 @@ typedef Vec3SGrid           Vec3fGrid;
 typedef Vec3DGrid           Vec3dGrid;
 typedef FloatGrid           ScalarGrid;
 typedef Vec3fGrid           VectorGrid;
-
+typedef MaskGrid            TopologyGrid;
 
 /// Global registration of basic types
 OPENVDB_API void initialize();
@@ -91,8 +92,8 @@ OPENVDB_API void uninitialize();
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-#endif // OPENVDB_INIT_HAS_BEEN_INCLUDED
+#endif // OPENVDB_OPENVDB_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2014 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
