@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -35,6 +35,9 @@
 #include <boost/python/exception_translator.hpp>
 #ifdef PY_OPENVDB_USE_NUMPY
 #define PY_ARRAY_UNIQUE_SYMBOL PY_OPENVDB_ARRAY_API
+#ifdef NPY_1_7_API_VERSION
+#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
+#endif
 #include <arrayobject.h> // for import_array()
 #endif
 #include "openvdb/openvdb.h"
@@ -297,12 +300,12 @@ struct MetaMapConverter
             if (py::extract<std::string>(val).check()) {
                 value.reset(
                     new StringMetadata(py::extract<std::string>(val)));
-            } else if (PyInt_Check(val.ptr())
-                && PyInt_AsLong(val.ptr()) <= std::numeric_limits<Int32>::max()
-                && PyInt_AsLong(val.ptr()) >= std::numeric_limits<Int32>::min())
+            } else if (PyLong_Check(val.ptr())
+                && PyLong_AsLong(val.ptr()) <= std::numeric_limits<Int32>::max()
+                && PyLong_AsLong(val.ptr()) >= std::numeric_limits<Int32>::min())
             {
                 value.reset(new Int32Metadata(py::extract<Int32>(val)));
-            } else if (PyInt_Check(val.ptr()) || PyLong_Check(val.ptr())) {
+            } else if (PyLong_Check(val.ptr()) || PyLong_Check(val.ptr())) {
                 value.reset(new Int64Metadata(py::extract<Int64>(val)));
             //} else if (py::extract<float>(val).check()) {
             //    value.reset(new FloatMetadata(py::extract<float>(val)));
@@ -697,6 +700,6 @@ BOOST_PYTHON_MODULE(PY_OPENVDB_MODULE_NAME)
 
 } // BOOST_PYTHON_MODULE
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
