@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -80,19 +80,31 @@ public:
     typedef typename TrackerT::LeafRange       LeafRange;
     typedef typename TrackerT::LeafType        LeafType;
     typedef typename TrackerT::BufferType      BufferType;
-    typedef typename TrackerT::ValueType       ScalarType;
+    typedef typename TrackerT::ValueType       ValueType;
 
     /// Main constructor
-    LevelSetMorphing(GridT& sourceGrid, const GridT& targetGrid, InterruptT* interrupt = NULL):
-        mTracker(sourceGrid, interrupt), mTarget(&targetGrid),
-        mSpatialScheme(math::HJWENO5_BIAS),
-        mTemporalScheme(math::TVD_RK2) {}
+    LevelSetMorphing(GridT& sourceGrid,
+                     const GridT& targetGrid,
+                     InterruptT* interrupt = NULL)
+        : mTracker(sourceGrid, interrupt)
+        , mTarget(&targetGrid)
+        , mMask(NULL)
+        , mSpatialScheme(math::HJWENO5_BIAS)
+        , mTemporalScheme(math::TVD_RK2)
+        , mMinMask(0)
+        , mDeltaMask(1)
+        , mInvertMask(false)
+    {
+    }
 
-    virtual ~LevelSetMorphing() {};
+    virtual ~LevelSetMorphing() {}
 
     /// Redefine the target level set
     void setTarget(const GridT& targetGrid) { mTarget = &targetGrid; }
-    
+
+    /// Define the alpha mask
+    void setAlphaMask(const GridT& maskGrid) { mMask = &maskGrid; }
+
     /// Return the spatial finite-difference scheme
     math::BiasedGradientScheme getSpatialScheme() const { return mSpatialScheme; }
     /// Set the spatial finite-difference scheme
@@ -134,35 +146,86 @@ public:
     /// @note A grain size of 0 or less disables multithreading!
     void setGrainSize(int grainsize) { mTracker.setGrainSize(grainsize); }
 
+    /// @brief Return the minimum value of the mask to be used for the
+    /// derivation of a smooth alpha value.
+    ValueType minMask() const { return mMinMask; }
+
+    /// @brief Return the maximum value of the mask to be used for the
+    /// derivation of a smooth alpha value.
+    ValueType maxMask() const { return mDeltaMask + mMinMask; }
+
+    /// @brief Define the range for the (optional) scalar mask.
+    /// @param min Minimum value of the range.
+    /// @param max Maximum value of the range.
+    /// @details Mask values outside the range maps to alpha values of
+    /// respectfully zero and one, and values inside the range maps
+    /// smoothly to 0->1 (unless of course the mask is inverted).
+    /// @throw ValueError if @a min is not smaller than @a max.
+    void setMaskRange(ValueType min, ValueType max)
+    {
+        if (!(min < max)) OPENVDB_THROW(ValueError, "Invalid mask range (expects min < max)");
+        mMinMask   = min;
+        mDeltaMask = max-min;
+    }
+
+    /// @brief Return true if the mask is inverted, i.e. min->max in the
+    /// original mask maps to 1->0 in the inverted alpha mask.
+    bool isMaskInverted() const { return mInvertMask; }
+    /// @brief Invert the optional mask, i.e. min->max in the original
+    /// mask maps to 1->0 in the inverted alpha mask.
+    void invertMask(bool invert=true) { mInvertMask = invert; }
+
     /// @brief Advect the level set from its current time, @a time0, to its
     /// final time, @a time1. If @a time0 > @a time1, perform backward advection.
     ///
     /// @return the number of CFL iterations used to advect from @a time0 to @a time1
-    size_t advect(ScalarType time0, ScalarType time1);
+    size_t advect(ValueType time0, ValueType time1);
 
 private:
 
+    // disallow copy construction and copy by assignment!
+    LevelSetMorphing(const LevelSetMorphing&);// not implemented
+    LevelSetMorphing& operator=(const LevelSetMorphing&);// not implemented
+
+    template<math::BiasedGradientScheme SpatialScheme>
+    size_t advect1(ValueType time0, ValueType time1);
+
+    template<math::BiasedGradientScheme SpatialScheme,
+             math::TemporalIntegrationScheme TemporalScheme>
+    size_t advect2(ValueType time0, ValueType time1);
+
+    template<math::BiasedGradientScheme SpatialScheme,
+             math::TemporalIntegrationScheme TemporalScheme,
+             typename MapType>
+    size_t advect3(ValueType time0, ValueType time1);
+
+    TrackerT                        mTracker;
+    const GridT                    *mTarget, *mMask;
+    math::BiasedGradientScheme      mSpatialScheme;
+    math::TemporalIntegrationScheme mTemporalScheme;
+    ValueType                       mMinMask, mDeltaMask;
+    bool                            mInvertMask;
+
     // This templated private class implements all the level set magic.
     template<typename MapT, math::BiasedGradientScheme SpatialScheme,
              math::TemporalIntegrationScheme TemporalScheme>
-    class LevelSetMorph
+    struct Morph
     {
-    public:
         /// Main constructor
-        LevelSetMorph(TrackerT& tracker, const GridT* target);
+        Morph(LevelSetMorphing<GridT, InterruptT>& parent);
         /// Shallow copy constructor called by tbb::parallel_for() threads
-        LevelSetMorph(const LevelSetMorph& other);
+        Morph(const Morph& other);
         /// Shallow copy constructor called by tbb::parallel_reduce() threads
-        LevelSetMorph(LevelSetMorph& other, tbb::split);
+        Morph(Morph& other, tbb::split);
         /// destructor
-        virtual ~LevelSetMorph() {}
-        /// Advect the level set from it's current time, time0, to it's final time, time1.
+        virtual ~Morph() {}
+        /// Advect the level set from its current time, time0, to its final time, time1.
         /// @return number of CFL iterations
-        size_t advect(ScalarType time0, ScalarType time1);
+        size_t advect(ValueType time0, ValueType time1);
         /// Used internally by tbb::parallel_for()
         void operator()(const LeafRange& r) const
         {
-            if (mTask) mTask(const_cast<LevelSetMorph*>(this), r);
+            if (mTask) mTask(const_cast<Morph*>(this), r);
             else OPENVDB_THROW(ValueError, "task is undefined - don\'t call this method directly");
         }
         /// Used internally by tbb::parallel_reduce()
@@ -172,14 +235,7 @@ private:
             else OPENVDB_THROW(ValueError, "task is undefined - don\'t call this method directly");
         }
         /// This is only called by tbb::parallel_reduce() threads
-        void join(const LevelSetMorph& other) { mMaxAbsS = math::Max(mMaxAbsS, other.mMaxAbsS); }
-    private:
-        typedef typename boost::function<void (LevelSetMorph*, const LeafRange&)> FuncType;
-        TrackerT*    mTracker;
-        const GridT* mTarget;
-        ScalarType   mMinAbsS, mMaxAbsS;
-        const MapT*  mMap;
-        FuncType     mTask;
+        void join(const Morph& other) { mMaxAbsS = math::Max(mMaxAbsS, other.mMaxAbsS); }
 
         /// Enum to define the type of multithreading
         enum ThreadingMode { PARALLEL_FOR, PARALLEL_REDUCE }; // for internal use
@@ -187,56 +243,41 @@ private:
         void cook(ThreadingMode mode, size_t swapBuffer = 0);
 
         /// Sample field and return the CFT time step
-        typename GridT::ValueType sampleSpeed(
-            ScalarType time0, ScalarType time1, Index speedBuffer);
+        typename GridT::ValueType sampleSpeed(ValueType time0, ValueType time1, Index speedBuffer);
         void sampleXformedSpeed(const LeafRange& r, Index speedBuffer);
         void sampleAlignedSpeed(const LeafRange& r, Index speedBuffer);
 
-        // Forward Euler advection steps: Phi(result) = Phi(0) - dt * Speed(speed)*|Grad[Phi(0)]|;
-        void euler1(const LeafRange& r, ScalarType dt, Index resultBuffer, Index speedBuffer);
-
         // Convex combination of Phi and a forward Euler advection steps:
         // Phi(result) = alpha * Phi(phi) + (1-alpha) * (Phi(0) - dt * Speed(speed)*|Grad[Phi(0)]|);
-        void euler2(const LeafRange& r, ScalarType dt, ScalarType alpha,
-                    Index phiBuffer, Index resultBuffer, Index speedBuffer);
+        template <int Nominator, int Denominator>
+        void euler(const LeafRange&, ValueType, Index, Index, Index);
+        inline void euler01(const LeafRange& r, ValueType t, Index s) {this->euler<0,1>(r,t,0,1,s);}
+        inline void euler12(const LeafRange& r, ValueType t) {this->euler<1,2>(r, t, 1, 1, 2);}
+        inline void euler34(const LeafRange& r, ValueType t) {this->euler<3,4>(r, t, 1, 2, 3);}
+        inline void euler13(const LeafRange& r, ValueType t) {this->euler<1,3>(r, t, 1, 2, 3);}
 
-    }; // end of private LevelSetMorph class
-
-    template<math::BiasedGradientScheme SpatialScheme>
-    size_t advect1(ScalarType time0, ScalarType time1);
-
-    template<math::BiasedGradientScheme SpatialScheme,
-             math::TemporalIntegrationScheme TemporalScheme>
-    size_t advect2(ScalarType time0, ScalarType time1);
-
-    template<math::BiasedGradientScheme SpatialScheme,
-             math::TemporalIntegrationScheme TemporalScheme,
-             typename MapType>
-    size_t advect3(ScalarType time0, ScalarType time1);
-
-    TrackerT                        mTracker;
-    const GridT*                    mTarget;
-    math::BiasedGradientScheme      mSpatialScheme;
-    math::TemporalIntegrationScheme mTemporalScheme;
-
-    // disallow copy by assignment
-    void operator=(const LevelSetMorphing& other) {}
+        typedef typename boost::function<void (Morph*, const LeafRange&)> FuncType;
+        LevelSetMorphing* mParent;
+        ValueType         mMinAbsS, mMaxAbsS;
+        const MapT*       mMap;
+        FuncType          mTask;
+    }; // end of private Morph struct
 
 };//end of LevelSetMorphing
 
 template<typename GridT, typename InterruptT>
 inline size_t
-LevelSetMorphing<GridT, InterruptT>::advect(ScalarType time0, ScalarType time1)
+LevelSetMorphing<GridT, InterruptT>::advect(ValueType time0, ValueType time1)
 {
     switch (mSpatialScheme) {
     case math::FIRST_BIAS:
         return this->advect1<math::FIRST_BIAS  >(time0, time1);
-        //case math::SECOND_BIAS:
-        //return this->advect1<math::SECOND_BIAS >(time0, time1);
-        //case math::THIRD_BIAS:
-        //return this->advect1<math::THIRD_BIAS  >(time0, time1);
-        //case math::WENO5_BIAS:
-        //return this->advect1<math::WENO5_BIAS  >(time0, time1);
+    //case math::SECOND_BIAS:
+    //return this->advect1<math::SECOND_BIAS >(time0, time1);
+    //case math::THIRD_BIAS:
+    //return this->advect1<math::THIRD_BIAS  >(time0, time1);
+    //case math::WENO5_BIAS:
+    //return this->advect1<math::WENO5_BIAS  >(time0, time1);
     case math::HJWENO5_BIAS:
         return this->advect1<math::HJWENO5_BIAS>(time0, time1);
     default:
@@ -248,7 +289,7 @@ LevelSetMorphing<GridT, InterruptT>::advect(ScalarType time0, ScalarType time1)
 template<typename GridT, typename InterruptT>
 template<math::BiasedGradientScheme SpatialScheme>
 inline size_t
-LevelSetMorphing<GridT, InterruptT>::advect1(ScalarType time0, ScalarType time1)
+LevelSetMorphing<GridT, InterruptT>::advect1(ValueType time0, ValueType time1)
 {
     switch (mTemporalScheme) {
     case math::TVD_RK1:
@@ -267,7 +308,7 @@ template<typename GridT, typename InterruptT>
 template<math::BiasedGradientScheme SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline size_t
-LevelSetMorphing<GridT, InterruptT>::advect2(ScalarType time0, ScalarType time1)
+LevelSetMorphing<GridT, InterruptT>::advect2(ValueType time0, ValueType time1)
 {
     const math::Transform& trans = mTracker.grid().transform();
     if (trans.mapType() == math::UniformScaleMap::mapType()) {
@@ -290,28 +331,26 @@ template<math::BiasedGradientScheme SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme,
          typename MapT>
 inline size_t
-LevelSetMorphing<GridT, InterruptT>::advect3(ScalarType time0, ScalarType time1)
+LevelSetMorphing<GridT, InterruptT>::advect3(ValueType time0, ValueType time1)
 {
-    LevelSetMorph<MapT, SpatialScheme, TemporalScheme> tmp(mTracker, mTarget);
+    Morph<MapT, SpatialScheme, TemporalScheme> tmp(*this);
     return tmp.advect(time0, time1);
 }
 
 
 ///////////////////////////////////////////////////////////////////////
 
-
 template<typename GridT, typename InterruptT>
 template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-LevelSetMorph(TrackerT& tracker, const GridT* target):
-    mTracker(&tracker),
-    mTarget(target),
-    mMinAbsS(1e-6),
-    mMap(tracker.grid().transform().template constMap<MapT>().get()),
-    mTask(0)
+Morph<MapT, SpatialScheme, TemporalScheme>::
+Morph(LevelSetMorphing<GridT, InterruptT>& parent)
+    : mParent(&parent)
+    , mMinAbsS(ValueType(1e-6))
+    , mMap(parent.mTracker.grid().transform().template constMap<MapT>().get())
+    , mTask(0)
 {
 }
 
@@ -320,14 +359,13 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-LevelSetMorph(const LevelSetMorph& other):
-    mTracker(other.mTracker),
-    mTarget(other.mTarget),
-    mMinAbsS(other.mMinAbsS),
-    mMaxAbsS(other.mMaxAbsS),
-    mMap(other.mMap),
-    mTask(other.mTask)
+Morph<MapT, SpatialScheme, TemporalScheme>::
+Morph(const Morph& other)
+    : mParent(other.mParent)
+    , mMinAbsS(other.mMinAbsS)
+    , mMaxAbsS(other.mMaxAbsS)
+    , mMap(other.mMap)
+    , mTask(other.mTask)
 {
 }
 
@@ -336,14 +374,13 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-LevelSetMorph(LevelSetMorph& other, tbb::split):
-    mTracker(other.mTracker),
-    mTarget(other.mTarget),
-    mMinAbsS(other.mMinAbsS),
-    mMaxAbsS(other.mMaxAbsS),
-    mMap(other.mMap),
-    mTask(other.mTask)
+Morph<MapT, SpatialScheme, TemporalScheme>::
+Morph(Morph& other, tbb::split)
+    : mParent(other.mParent)
+    , mMinAbsS(other.mMinAbsS)
+    , mMaxAbsS(other.mMaxAbsS)
+    , mMap(other.mMap)
+    , mTask(other.mTask)
 {
 }
 
@@ -352,17 +389,17 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline size_t
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-advect(ScalarType time0, ScalarType time1)
+Morph<MapT, SpatialScheme, TemporalScheme>::
+advect(ValueType time0, ValueType time1)
 {
     // Make sure we have enough temporal auxiliary buffers for the time
     // integration AS WELL AS an extra buffer with the speed function!
     static const Index auxBuffers = 1 + (TemporalScheme == math::TVD_RK3 ? 2 : 1);
     size_t countCFL = 0;
-    while (time0 < time1 && mTracker->checkInterrupter()) {
-        mTracker->leafs().rebuildAuxBuffers(auxBuffers);
+    while (time0 < time1 && mParent->mTracker.checkInterrupter()) {
+        mParent->mTracker.leafs().rebuildAuxBuffers(auxBuffers);
 
-        const ScalarType dt = this->sampleSpeed(time0, time1, auxBuffers);
+        const ValueType dt = this->sampleSpeed(time0, time1, auxBuffers);
         if ( math::isZero(dt) ) break;//V is essentially zero so terminate
 
         OPENVDB_NO_UNREACHABLE_CODE_WARNING_BEGIN //switch is resolved at compile-time
@@ -370,42 +407,45 @@ advect(ScalarType time0, ScalarType time1)
         case math::TVD_RK1:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * Speed(2) * |Grad[Phi(0)]|
-            mTask = boost::bind(&LevelSetMorph::euler1, _1, _2, dt, /*result=*/1, /*speed*/2);
+            mTask = boost::bind(&Morph::euler01, _1, _2, dt, /*speed*/2);
+
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
             break;
         case math::TVD_RK2:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * Speed(2) * |Grad[Phi(0)]|
-            mTask = boost::bind(&LevelSetMorph::euler1, _1, _2, dt, /*result=*/1, /*speed*/2);
+            mTask = boost::bind(&Morph::euler01, _1, _2, dt, /*speed*/2);
+
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt
             // Phi_t2(1) = 1/2 * Phi_t0(1) + 1/2 * (Phi_t1(0) - dt * Speed(2) * |Grad[Phi(0)]|)
-            mTask = boost::bind(&LevelSetMorph::euler2, _1, _2, dt, ScalarType(0.5),
-                                /*phi=*/1, /*result=*/1, /*speed*/2);
+            mTask = boost::bind(&Morph::euler12, _1, _2, dt);
+
             // Cook and swap buffer 0 and 1 such that Phi_t2(0) and Phi_t1(1)
             this->cook(PARALLEL_FOR, 1);
             break;
         case math::TVD_RK3:
             // Perform one explicit Euler step: t1 = t0 + dt
             // Phi_t1(1) = Phi_t0(0) - dt * Speed(3) * |Grad[Phi(0)]|
-            mTask = boost::bind(&LevelSetMorph::euler1, _1, _2, dt, /*result=*/1, /*speed*/3);
+            mTask = boost::bind(&Morph::euler01, _1, _2, dt, /*speed*/3);
+
             // Cook and swap buffer 0 and 1 such that Phi_t1(0) and Phi_t0(1)
             this->cook(PARALLEL_FOR, 1);
 
             // Convex combine explict Euler step: t2 = t0 + dt/2
             // Phi_t2(2) = 3/4 * Phi_t0(1) + 1/4 * (Phi_t1(0) - dt * Speed(3) * |Grad[Phi(0)]|)
-            mTask = boost::bind(&LevelSetMorph::euler2, _1, _2, dt, ScalarType(0.75),
-                                /*phi=*/1, /*result=*/2, /*speed*/3);
+            mTask = boost::bind(&Morph::euler34, _1, _2, dt);
+
             // Cook and swap buffer 0 and 2 such that Phi_t2(0) and Phi_t1(2)
             this->cook(PARALLEL_FOR, 2);
 
             // Convex combine explict Euler step: t3 = t0 + dt
             // Phi_t3(2) = 1/3 * Phi_t0(1) + 2/3 * (Phi_t2(0) - dt * Speed(3) * |Grad[Phi(0)]|)
-            mTask = boost::bind(&LevelSetMorph::euler2, _1, _2, dt, ScalarType(1.0/3.0),
-                                /*phi=*/1, /*result=*/2, /*speed*/3);
+            mTask = boost::bind(&Morph::euler13, _1, _2, dt);
+
             // Cook and swap buffer 0 and 2 such that Phi_t3(0) and Phi_t2(2)
             this->cook(PARALLEL_FOR, 2);
             break;
@@ -416,10 +456,10 @@ advect(ScalarType time0, ScalarType time1)
 
         time0 += dt;
         ++countCFL;
-        mTracker->leafs().removeAuxBuffers();
+        mParent->mTracker.leafs().removeAuxBuffers();
 
         // Track the narrow band
-        mTracker->track();
+        mParent->mTracker.track();
     }//end wile-loop over time
 
     return countCFL;//number of CLF propagation steps
@@ -430,25 +470,27 @@ template<typename MapT, math::BiasedGradientScheme SpatialScheme,
          math::TemporalIntegrationScheme TemporalScheme>
 inline typename GridT::ValueType
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-sampleSpeed(ScalarType time0, ScalarType time1, Index speedBuffer)
+Morph<MapT, SpatialScheme, TemporalScheme>::
+sampleSpeed(ValueType time0, ValueType time1, Index speedBuffer)
 {
     mMaxAbsS = mMinAbsS;
-    const size_t leafCount = mTracker->leafs().leafCount();
-    if (leafCount==0 || time0 >= time1) return ScalarType(0);
+    const size_t leafCount = mParent->mTracker.leafs().leafCount();
+    if (leafCount==0 || time0 >= time1) return ValueType(0);
 
-    if (mTarget->transform() == mTracker->grid().transform()) {
-        mTask = boost::bind(&LevelSetMorph::sampleAlignedSpeed, _1, _2, speedBuffer);
+    const math::Transform& xform  = mParent->mTracker.grid().transform();
+    if (mParent->mTarget->transform() == xform &&
+        (mParent->mMask == NULL || mParent->mMask->transform() == xform)) {
+        mTask = boost::bind(&Morph::sampleAlignedSpeed, _1, _2, speedBuffer);
     } else {
-        mTask = boost::bind(&LevelSetMorph::sampleXformedSpeed, _1, _2, speedBuffer);
+        mTask = boost::bind(&Morph::sampleXformedSpeed, _1, _2, speedBuffer);
     }
     this->cook(PARALLEL_REDUCE);
-    if (math::isApproxEqual(mMinAbsS, mMaxAbsS)) return ScalarType(0);//speed is essentially zero
-    static const ScalarType CFL = (TemporalScheme == math::TVD_RK1 ? ScalarType(0.3) :
-                                   TemporalScheme == math::TVD_RK2 ? ScalarType(0.9) :
-                                   ScalarType(1.0))/math::Sqrt(ScalarType(3.0));
-    const ScalarType dt = math::Abs(time1 - time0), dx = mTracker->voxelSize();
-    return math::Min(dt, ScalarType(CFL*dx/mMaxAbsS));
+    if (math::isApproxEqual(mMinAbsS, mMaxAbsS)) return ValueType(0);//speed is essentially zero
+    static const ValueType CFL = (TemporalScheme == math::TVD_RK1 ? ValueType(0.3) :
+                                  TemporalScheme == math::TVD_RK2 ? ValueType(0.9) :
+                                  ValueType(1.0))/math::Sqrt(ValueType(3.0));
+    const ValueType dt = math::Abs(time1 - time0), dx = mParent->mTracker.voxelSize();
+    return math::Min(dt, ValueType(CFL*dx/mMaxAbsS));
 }
 
 template<typename GridT, typename InterruptT>
@@ -456,21 +498,46 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
+Morph<MapT, SpatialScheme, TemporalScheme>::
 sampleXformedSpeed(const LeafRange& range, Index speedBuffer)
 {
     typedef typename LeafType::ValueOnCIter VoxelIterT;
     typedef tools::GridSampler<typename GridT::ConstAccessor, tools::BoxSampler> SamplerT;
     const MapT& map = *mMap;
-    mTracker->checkInterrupter();
+    mParent->mTracker.checkInterrupter();
 
-    SamplerT sampler(mTarget->getAccessor(),  mTarget->transform());
-    for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
-        BufferType& speed = leafIter.buffer(speedBuffer);
-        for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
-            ScalarType& s = const_cast<ScalarType&>(speed.getValue(voxelIter.pos()));
-            s -= sampler.wsSample(map.applyMap(voxelIter.getCoord().asVec3d()));
-            mMaxAbsS = math::Max(mMaxAbsS, math::Abs(s));
+    typename GridT::ConstAccessor targetAcc = mParent->mTarget->getAccessor();
+    SamplerT target(targetAcc, mParent->mTarget->transform());
+    if (mParent->mMask == NULL) {
+        for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+            ValueType* speed = leafIter.buffer(speedBuffer).data();
+            bool isZero = true;
+            for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
+                ValueType& s = speed[voxelIter.pos()];
+                s -= target.wsSample(map.applyMap(voxelIter.getCoord().asVec3d()));
+                if (!math::isApproxZero(s)) isZero = false;
+                mMaxAbsS = math::Max(mMaxAbsS, math::Abs(s));
+            }
+            if (isZero) speed[0] = std::numeric_limits<ValueType>::max();//tag first voxel
+        }
+    } else {
+        const ValueType min = mParent->mMinMask, invNorm = 1.0f/(mParent->mDeltaMask);
+        const bool invMask = mParent->isMaskInverted();
+        typename GridT::ConstAccessor maskAcc = mParent->mMask->getAccessor();
+        SamplerT mask(maskAcc,  mParent->mMask->transform());
+        for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+            ValueType* speed = leafIter.buffer(speedBuffer).data();
+            bool isZero = true;
+            for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
+                const Vec3R xyz = map.applyMap(voxelIter.getCoord().asVec3d());//world space
+                const ValueType a = math::SmoothUnitStep((mask.wsSample(xyz)-min)*invNorm);
+                ValueType& s = speed[voxelIter.pos()];
+                s -= target.wsSample(xyz);
+                s *= invMask ? 1 - a : a;
+                if (!math::isApproxZero(s)) isZero = false;
+                mMaxAbsS = math::Max(mMaxAbsS, math::Abs(s));
+            }
+            if (isZero) speed[0] = std::numeric_limits<ValueType>::max();//tag first voxel
         }
     }
 }
@@ -480,19 +547,43 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
+Morph<MapT, SpatialScheme, TemporalScheme>::
 sampleAlignedSpeed(const LeafRange& range, Index speedBuffer)
 {
     typedef typename LeafType::ValueOnCIter VoxelIterT;
-    mTracker->checkInterrupter();
+    mParent->mTracker.checkInterrupter();
 
-    typename GridT::ConstAccessor target = mTarget->getAccessor();
-    for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
-        BufferType& speed = leafIter.buffer(speedBuffer);
-        for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
-            ScalarType& s = const_cast<ScalarType&>(speed.getValue(voxelIter.pos()));
-            s -= target.getValue(voxelIter.getCoord());
-            mMaxAbsS = math::Max(mMaxAbsS, math::Abs(s));
+    typename GridT::ConstAccessor target = mParent->mTarget->getAccessor();
+
+    if (mParent->mMask == NULL) {
+        for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+            ValueType* speed = leafIter.buffer(speedBuffer).data();
+            bool isZero = true;
+            for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
+                ValueType& s = speed[voxelIter.pos()];
+                s -= target.getValue(voxelIter.getCoord());
+                if (!math::isApproxZero(s)) isZero = false;
+                mMaxAbsS = math::Max(mMaxAbsS, math::Abs(s));
+            }
+            if (isZero) speed[0] = std::numeric_limits<ValueType>::max();//tag first voxel
+        }
+    } else {
+        const ValueType min = mParent->mMinMask, invNorm = 1.0f/(mParent->mDeltaMask);
+        const bool invMask = mParent->isMaskInverted();
+        typename GridT::ConstAccessor mask = mParent->mMask->getAccessor();
+        for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
+            ValueType* speed = leafIter.buffer(speedBuffer).data();
+            bool isZero = true;
+            for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
+                const Coord ijk = voxelIter.getCoord();//index space
+                const ValueType a = math::SmoothUnitStep((mask.getValue(ijk)-min)*invNorm);
+                ValueType& s = speed[voxelIter.pos()];
+                s -= target.getValue(ijk);
+                s *= invMask ? 1 - a : a;
+                if (!math::isApproxZero(s)) isZero = false;
+                mMaxAbsS = math::Max(mMaxAbsS, math::Abs(s));
+            }
+            if (isZero) speed[0] = std::numeric_limits<ValueType>::max();//tag first voxel
         }
     }
 }
@@ -502,15 +593,15 @@ template <typename MapT, math::BiasedGradientScheme SpatialScheme,
           math::TemporalIntegrationScheme TemporalScheme>
 inline void
 LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
+Morph<MapT, SpatialScheme, TemporalScheme>::
 cook(ThreadingMode mode, size_t swapBuffer)
 {
-    mTracker->startInterrupter("Morphing level set");
+    mParent->mTracker.startInterrupter("Morphing level set");
 
-    const int grainSize   = mTracker->getGrainSize();
-    const LeafRange range = mTracker->leafs().leafRange(grainSize);
+    const int grainSize   = mParent->mTracker.getGrainSize();
+    const LeafRange range = mParent->mTracker.leafs().leafRange(grainSize);
 
-    if (mTracker->getGrainSize()==0) {
+    if (mParent->mTracker.getGrainSize()==0) {
         (*this)(range);
     } else if (mode == PARALLEL_FOR) {
         tbb::parallel_for(range, *this);
@@ -520,74 +611,46 @@ cook(ThreadingMode mode, size_t swapBuffer)
         throw std::runtime_error("Undefined threading mode");
     }
 
-    mTracker->leafs().swapLeafBuffer(swapBuffer, grainSize == 0);
+    mParent->mTracker.leafs().swapLeafBuffer(swapBuffer, grainSize == 0);
 
-    mTracker->endInterrupter();
+    mParent->mTracker.endInterrupter();
 }
 
-// Forward Euler advection steps:
-// Phi(result) = Phi(0) - dt * Phi(speed) * |Grad[Phi(0)]|
 template<typename GridT, typename InterruptT>
-template <typename MapT,
-          math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
+template<typename MapT, math::BiasedGradientScheme SpatialScheme,
+         math::TemporalIntegrationScheme TemporalScheme>
+template <int Nominator, int Denominator>
 inline void
-LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-euler1(const LeafRange& range, ScalarType dt, Index resultBuffer, Index speedBuffer)
+LevelSetMorphing<GridT,InterruptT>::
+Morph<MapT, SpatialScheme, TemporalScheme>::
+euler(const LeafRange& range, ValueType dt,
+      Index phiBuffer, Index resultBuffer, Index speedBuffer)
 {
-    typedef math::BIAS_SCHEME<SpatialScheme>                             Scheme;
-    typedef typename Scheme::template ISStencil<GridType>::StencilType   Stencil;
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
+    typedef math::BIAS_SCHEME<SpatialScheme>                             SchemeT;
+    typedef typename SchemeT::template ISStencil<GridType>::StencilType  StencilT;
+    typedef typename LeafType::ValueOnCIter                              VoxelIterT;
+    typedef math::GradientNormSqrd<MapT, SpatialScheme>                  NumGrad;
 
-    mTracker->checkInterrupter();
+    static const ValueType Alpha = ValueType(Nominator)/ValueType(Denominator);
+    static const ValueType Beta  = ValueType(1) - Alpha;
+
+    mParent->mTracker.checkInterrupter();
     const MapT& map = *mMap;
-    Stencil stencil(mTracker->grid());
+    StencilT stencil(mParent->mTracker.grid());
 
     for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
-        BufferType& speed  = leafIter.buffer(speedBuffer);
-        BufferType& result = leafIter.buffer(resultBuffer);
+        const ValueType* speed = leafIter.buffer(speedBuffer).data();
+        if (math::isExactlyEqual(speed[0], std::numeric_limits<ValueType>::max())) continue;
+        const ValueType* phi = leafIter.buffer(phiBuffer).data();
+        ValueType* result = leafIter.buffer(resultBuffer).data();
         for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
             const Index n = voxelIter.pos();
+            if (math::isApproxZero(speed[n])) continue;
             stencil.moveTo(voxelIter);
-            const ScalarType G = math::GradientNormSqrd<MapT,SpatialScheme>::result(map, stencil);
-            result.setValue(n, *voxelIter - dt * speed.getValue(n) * G);
-        }
-    }
-}
-
-// Convex combination of Phi and a forward Euler advection steps:
-// Phi(result) = alpha * Phi(phi) + (1-alpha) * (Phi(0) - dt * Phi(speed) * |Grad[Phi(0)]|)
-template<typename GridT, typename InterruptT>
-template <typename MapT, math::BiasedGradientScheme SpatialScheme,
-          math::TemporalIntegrationScheme TemporalScheme>
-inline void
-LevelSetMorphing<GridT, InterruptT>::
-LevelSetMorph<MapT, SpatialScheme, TemporalScheme>::
-euler2(const LeafRange& range, ScalarType dt, ScalarType alpha,
-       Index phiBuffer, Index resultBuffer, Index speedBuffer)
-{
-    typedef math::BIAS_SCHEME<SpatialScheme>                             Scheme;
-    typedef typename Scheme::template ISStencil<GridType>::StencilType   Stencil;
-    typedef typename LeafType::ValueOnCIter VoxelIterT;
-
-    mTracker->checkInterrupter();
-    const MapT& map = *mMap;
-    const ScalarType beta = ScalarType(1.0) - alpha;
-    Stencil stencil(mTracker->grid());
-
-    for (typename LeafRange::Iterator leafIter = range.begin(); leafIter; ++leafIter) {
-        BufferType& speed  = leafIter.buffer(speedBuffer);
-        BufferType& result = leafIter.buffer(resultBuffer);
-        BufferType& phi    = leafIter.buffer(phiBuffer);
-        for (VoxelIterT voxelIter = leafIter->cbeginValueOn(); voxelIter; ++voxelIter) {
-            const Index n = voxelIter.pos();
-            stencil.moveTo(voxelIter);
-            const ScalarType G = math::GradientNormSqrd<MapT,SpatialScheme>::result(map, stencil);
-            result.setValue(n,
-                alpha * phi.getValue(n) + beta * (*voxelIter - dt * speed.getValue(n) * G));
-        }
-    }
+            const ValueType v = stencil.getValue() - dt * speed[n] * NumGrad::result(map, stencil);
+            result[n] = Nominator ? Alpha * phi[n] + Beta * v : v;
+        }//loop over active voxels in the leaf of the mask
+    }//loop over leafs of the level set
 }
 
 } // namespace tools
@@ -596,6 +659,6 @@ euler2(const LeafRange& range, ScalarType dt, ScalarType alpha,
 
 #endif // OPENVDB_TOOLS_LEVEL_SET_MORPH_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
