@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -123,15 +123,16 @@ fillWithSpheres(
 
 
 /// @brief  Accelerated closest surface point queries for narrow band level sets.
-///         Supports queries that originate at arbitrary worldspace locations, is
+///         Supports queries that originate at arbitrary world-space locations, is
 ///         not confined to the narrow band region of the input volume geometry.
 template<typename GridT>
 class ClosestSurfacePoint
 {
 public:
     typedef typename GridT::TreeType TreeT;
-    typedef typename TreeT::template ValueConverter<int>::Type IntTreeT;
-    typedef typename TreeT::template ValueConverter<Int16>::Type Int16TreeT;
+    typedef typename TreeT::template ValueConverter<bool>::Type     BoolTreeT;
+    typedef typename TreeT::template ValueConverter<Index32>::Type  Index32TreeT;
+    typedef typename TreeT::template ValueConverter<Int16>::Type    Int16TreeT;
 
 
     ClosestSurfacePoint();
@@ -179,22 +180,22 @@ public:
 
     /// @{
     /// @brief Tree accessors
-    const IntTreeT& indexTree() const { return *mIdxTreePt; }
+    const Index32TreeT& indexTree() const { return *mIdxTreePt; }
     const Int16TreeT& signTree() const { return *mSignTreePt; }
     /// @}
 
 private:
-    typedef typename IntTreeT::LeafNodeType IntLeafT;
+    typedef typename Index32TreeT::LeafNodeType Index32LeafT;
     typedef std::pair<size_t, size_t> IndexRange;
 
     bool mIsInitialized;
     std::vector<Vec4R> mLeafBoundingSpheres, mNodeBoundingSpheres;
     std::vector<IndexRange> mLeafRanges;
-    std::vector<const IntLeafT*> mLeafNodes;
+    std::vector<const Index32LeafT*> mLeafNodes;
     PointList mSurfacePointList;
     size_t mPointListSize, mMaxNodeLeafs;
     float mMaxRadiusSqr;
-    typename IntTreeT::Ptr mIdxTreePt;
+    typename Index32TreeT::Ptr mIdxTreePt;
     typename Int16TreeT::Ptr mSignTreePt;
 
     bool search(std::vector<Vec3R>&, std::vector<float>&, bool transformPoints);
@@ -227,13 +228,13 @@ private:
 };
 
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 class LeafBS
 {
 public:
 
     LeafBS(std::vector<Vec4R>& leafBoundingSpheres,
-        const std::vector<const IntLeafT*>& leafNodes,
+        const std::vector<const Index32LeafT*>& leafNodes,
         const math::Transform& transform,
         const PointList& surfacePointList);
 
@@ -244,15 +245,15 @@ public:
 
 private:
     std::vector<Vec4R>& mLeafBoundingSpheres;
-    const std::vector<const IntLeafT*>& mLeafNodes;
+    const std::vector<const Index32LeafT*>& mLeafNodes;
     const math::Transform& mTransform;
     const PointList& mSurfacePointList;
 };
 
-template<typename IntLeafT>
-LeafBS<IntLeafT>::LeafBS(
+template<typename Index32LeafT>
+LeafBS<Index32LeafT>::LeafBS(
     std::vector<Vec4R>& leafBoundingSpheres,
-    const std::vector<const IntLeafT*>& leafNodes,
+    const std::vector<const Index32LeafT*>& leafNodes,
     const math::Transform& transform,
     const PointList& surfacePointList)
     : mLeafBoundingSpheres(leafBoundingSpheres)
@@ -262,9 +263,9 @@ LeafBS<IntLeafT>::LeafBS(
 {
 }
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 void
-LeafBS<IntLeafT>::run(bool threaded)
+LeafBS<Index32LeafT>::run(bool threaded)
 {
     if (threaded) {
         tbb::parallel_for(tbb::blocked_range<size_t>(0, mLeafNodes.size()), *this);
@@ -273,11 +274,11 @@ LeafBS<IntLeafT>::run(bool threaded)
     }
 }
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 void
-LeafBS<IntLeafT>::operator()(const tbb::blocked_range<size_t>& range) const
+LeafBS<Index32LeafT>::operator()(const tbb::blocked_range<size_t>& range) const
 {
-    typename IntLeafT::ValueOnCIter iter;
+    typename Index32LeafT::ValueOnCIter iter;
     Vec3s avg;
 
     for (size_t n = range.begin(); n != range.end(); ++n) {
@@ -320,10 +321,9 @@ public:
         const std::vector<IndexRange>& leafRanges,
         const std::vector<Vec4R>& leafBoundingSpheres);
 
-    void run(bool threaded = true);
+    inline void run(bool threaded = true);
 
-
-    void operator()(const tbb::blocked_range<size_t>&) const;
+    inline void operator()(const tbb::blocked_range<size_t>&) const;
 
 private:
     std::vector<Vec4R>& mNodeBoundingSpheres;
@@ -354,7 +354,7 @@ NodeBS::run(bool threaded)
 inline void
 NodeBS::operator()(const tbb::blocked_range<size_t>& range) const
 {
-    Vec3s avg, pos;
+    Vec3d avg, pos;
 
     for (size_t n = range.begin(); n != range.end(); ++n) {
 
@@ -362,7 +362,7 @@ NodeBS::operator()(const tbb::blocked_range<size_t>& range) const
         avg[1] = 0.0;
         avg[2] = 0.0;
 
-        int count = mLeafRanges[n].second - mLeafRanges[n].first;
+        int count = int(mLeafRanges[n].second) - int(mLeafRanges[n].first);
 
         for (size_t i = mLeafRanges[n].first; i < mLeafRanges[n].second; ++i) {
             avg[0] += mLeafBoundingSpheres[i][0];
@@ -373,14 +373,14 @@ NodeBS::operator()(const tbb::blocked_range<size_t>& range) const
         if (count > 1) avg *= float(1.0 / double(count));
 
 
-        float maxDist = 0.0;
+        double maxDist = 0.0;
 
         for (size_t i = mLeafRanges[n].first; i < mLeafRanges[n].second; ++i) {
             pos[0] = mLeafBoundingSpheres[i][0];
             pos[1] = mLeafBoundingSpheres[i][1];
             pos[2] = mLeafBoundingSpheres[i][2];
 
-            float tmpDist = (pos - avg).lengthSqr() + mLeafBoundingSpheres[i][3];
+            double tmpDist = (pos - avg).lengthSqr() + mLeafBoundingSpheres[i][3];
             if (tmpDist > maxDist) maxDist = tmpDist;
         }
 
@@ -398,7 +398,7 @@ NodeBS::operator()(const tbb::blocked_range<size_t>& range) const
 ////////////////////////////////////////
 
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 class ClosestPointDist
 {
 public:
@@ -408,7 +408,7 @@ public:
         std::vector<Vec3R>& instancePoints,
         std::vector<float>& instanceDistances,
         const PointList& surfacePointList,
-        const std::vector<const IntLeafT*>& leafNodes,
+        const std::vector<const Index32LeafT*>& leafNodes,
         const std::vector<IndexRange>& leafRanges,
         const std::vector<Vec4R>& leafBoundingSpheres,
         const std::vector<Vec4R>& nodeBoundingSpheres,
@@ -423,7 +423,7 @@ public:
 
 private:
 
-    void evalLeaf(size_t index, const IntLeafT& leaf) const;
+    void evalLeaf(size_t index, const Index32LeafT& leaf) const;
     void evalNode(size_t pointIndex, size_t nodeIndex) const;
 
 
@@ -432,7 +432,7 @@ private:
 
     const PointList& mSurfacePointList;
 
-    const std::vector<const IntLeafT*>& mLeafNodes;
+    const std::vector<const Index32LeafT*>& mLeafNodes;
     const std::vector<IndexRange>& mLeafRanges;
     const std::vector<Vec4R>& mLeafBoundingSpheres;
     const std::vector<Vec4R>& mNodeBoundingSpheres;
@@ -444,12 +444,12 @@ private:
 };
 
 
-template<typename IntLeafT>
-ClosestPointDist<IntLeafT>::ClosestPointDist(
+template<typename Index32LeafT>
+ClosestPointDist<Index32LeafT>::ClosestPointDist(
     std::vector<Vec3R>& instancePoints,
     std::vector<float>& instanceDistances,
     const PointList& surfacePointList,
-    const std::vector<const IntLeafT*>& leafNodes,
+    const std::vector<const Index32LeafT*>& leafNodes,
     const std::vector<IndexRange>& leafRanges,
     const std::vector<Vec4R>& leafBoundingSpheres,
     const std::vector<Vec4R>& nodeBoundingSpheres,
@@ -470,9 +470,9 @@ ClosestPointDist<IntLeafT>::ClosestPointDist(
 }
 
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 void
-ClosestPointDist<IntLeafT>::run(bool threaded)
+ClosestPointDist<Index32LeafT>::run(bool threaded)
 {
     if (threaded) {
         tbb::parallel_for(tbb::blocked_range<size_t>(0, mInstancePoints.size()), *this);
@@ -481,11 +481,11 @@ ClosestPointDist<IntLeafT>::run(bool threaded)
     }
 }
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 void
-ClosestPointDist<IntLeafT>::evalLeaf(size_t index, const IntLeafT& leaf) const
+ClosestPointDist<Index32LeafT>::evalLeaf(size_t index, const Index32LeafT& leaf) const
 {
-    typename IntLeafT::ValueOnCIter iter;
+    typename Index32LeafT::ValueOnCIter iter;
     const Vec3s center = mInstancePoints[index];
     size_t& closestPointIndex = const_cast<size_t&>(mClosestPointIndex);
 
@@ -502,9 +502,9 @@ ClosestPointDist<IntLeafT>::evalLeaf(size_t index, const IntLeafT& leaf) const
 }
 
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 void
-ClosestPointDist<IntLeafT>::evalNode(size_t pointIndex, size_t nodeIndex) const
+ClosestPointDist<Index32LeafT>::evalNode(size_t pointIndex, size_t nodeIndex) const
 {
     const Vec3R& pos = mInstancePoints[pointIndex];
     float minDist = mInstanceDistances[pointIndex];
@@ -512,15 +512,16 @@ ClosestPointDist<IntLeafT>::evalNode(size_t pointIndex, size_t nodeIndex) const
     Vec3R center;
     bool updatedDist = false;
 
-    for (size_t i = mLeafRanges[nodeIndex].first, n = 0; i < mLeafRanges[nodeIndex].second; ++i, ++n) {
-
+    for (size_t i = mLeafRanges[nodeIndex].first, n = 0;
+        i < mLeafRanges[nodeIndex].second; ++i, ++n)
+    {
         float& distToLeaf = const_cast<float&>(mLeafDistances[n]);
 
         center[0] = mLeafBoundingSpheres[i][0];
         center[1] = mLeafBoundingSpheres[i][1];
         center[2] = mLeafBoundingSpheres[i][2];
 
-        distToLeaf = (pos - center).lengthSqr() - mLeafBoundingSpheres[i][3];
+        distToLeaf = float((pos - center).lengthSqr() - mLeafBoundingSpheres[i][3]);
 
         if (distToLeaf < minDist) {
             minDist = distToLeaf;
@@ -533,7 +534,9 @@ ClosestPointDist<IntLeafT>::evalNode(size_t pointIndex, size_t nodeIndex) const
 
     evalLeaf(pointIndex, *mLeafNodes[minDistIdx]);
 
-    for (size_t i = mLeafRanges[nodeIndex].first, n = 0; i < mLeafRanges[nodeIndex].second; ++i, ++n) {
+    for (size_t i = mLeafRanges[nodeIndex].first, n = 0;
+        i < mLeafRanges[nodeIndex].second; ++i, ++n)
+    {
         if (mLeafDistances[n] < mInstanceDistances[pointIndex] && i != minDistIdx) {
             evalLeaf(pointIndex, *mLeafNodes[i]);
         }
@@ -541,9 +544,9 @@ ClosestPointDist<IntLeafT>::evalNode(size_t pointIndex, size_t nodeIndex) const
 }
 
 
-template<typename IntLeafT>
+template<typename Index32LeafT>
 void
-ClosestPointDist<IntLeafT>::operator()(const tbb::blocked_range<size_t>& range) const
+ClosestPointDist<Index32LeafT>::operator()(const tbb::blocked_range<size_t>& range) const
 {
     Vec3R center;
     for (size_t n = range.begin(); n != range.end(); ++n) {
@@ -559,7 +562,7 @@ ClosestPointDist<IntLeafT>::operator()(const tbb::blocked_range<size_t>& range)
             center[1] = mNodeBoundingSpheres[i][1];
             center[2] = mNodeBoundingSpheres[i][2];
 
-            distToNode = (pos - center).lengthSqr() - mNodeBoundingSpheres[i][3];
+            distToNode = float((pos - center).lengthSqr() - mNodeBoundingSpheres[i][3]);
 
             if (distToNode < minDist) {
                 minDist = distToNode;
@@ -593,13 +596,13 @@ public:
         bool overlapping);
 
     float radius() const { return mRadius; }
-    int index() const { return mIndex; };
+    int index() const { return mIndex; }
 
-    void run(bool threaded = true);
+    inline void run(bool threaded = true);
 
 
     UpdatePoints(UpdatePoints&, tbb::split);
-    void operator()(const tbb::blocked_range<size_t>& range);
+    inline void operator()(const tbb::blocked_range<size_t>& range);
     void join(const UpdatePoints& rhs)
     {
         if (rhs.mRadius > mRadius) {
@@ -684,7 +687,7 @@ UpdatePoints::operator()(const tbb::blocked_range<size_t>& range)
 
         if (mDistances[n] > mRadius) {
             mRadius = mDistances[n];
-            mIndex = n;
+            mIndex = int(n);
         }
     }
 }
@@ -718,14 +721,9 @@ fillWithSpheres(
     typedef typename GridT::TreeType TreeT;
     typedef typename GridT::ValueType ValueT;
 
-    typedef typename TreeT::template ValueConverter<bool>::Type BoolTreeT;
-    typedef typename TreeT::template ValueConverter<int>::Type IntTreeT;
-    typedef typename TreeT::template ValueConverter<Int16>::Type Int16TreeT;
-
-    typedef tree::LeafManager<const TreeT> LeafManagerT;
-    typedef tree::LeafManager<IntTreeT>    IntLeafManagerT;
-    typedef tree::LeafManager<Int16TreeT>  Int16LeafManagerT;
-
+    typedef typename TreeT::template ValueConverter<bool>::Type     BoolTreeT;
+    typedef typename TreeT::template ValueConverter<Index32>::Type  Index32TreeT;
+    typedef typename TreeT::template ValueConverter<Int16>::Type    Int16TreeT;
 
     typedef boost::mt11213b RandGen;
     RandGen mtRand(/*seed=*/0);
@@ -753,8 +751,8 @@ fillWithSpheres(
         instancePoints.reserve(instances);
         internal::PointAccessor ptnAcc(instancePoints);
 
-        UniformPointScatter<internal::PointAccessor, RandGen, InterrupterT>
-            scatter(ptnAcc, (addNBPoints ? (instances / 2) : instances), mtRand, interrupter);
+        UniformPointScatter<internal::PointAccessor, RandGen, InterrupterT> scatter(
+            ptnAcc, Index64(addNBPoints ? (instances / 2) : instances), mtRand, interrupter);
 
         scatter(*interiorMaskPtr);
     }
@@ -767,21 +765,22 @@ fillWithSpheres(
     csp.initialize(grid, isovalue, interrupter);
 
     // add extra instance points in the interior narrow band.
-    if (instancePoints.size() < instances) {
+    if (instancePoints.size() < size_t(instances)) {
         const Int16TreeT& signTree = csp.signTree();
         typename Int16TreeT::LeafNodeType::ValueOnCIter it;
         typename Int16TreeT::LeafCIter leafIt = signTree.cbeginLeaf();
 
         for (; leafIt; ++leafIt) {
             for (it = leafIt->cbeginValueOn(); it; ++it) {
-                const int flags = it.getValue();
-                if (!(0xE00 & flags) && (flags & 0x100)) {
+
+                const int flags = int(it.getValue());
+                if (!(volume_to_mesh_internal::EDGES & flags) && (volume_to_mesh_internal::INSIDE & flags)) {
                     instancePoints.push_back(transform.indexToWorld(it.getCoord()));
                 }
 
-                if (instancePoints.size() == instances) break;
+                if (instancePoints.size() == size_t(instances)) break;
             }
-            if (instancePoints.size() == instances) break;
+            if (instancePoints.size() == size_t(instances)) break;
         }
     }
 
@@ -797,14 +796,14 @@ fillWithSpheres(
     for (size_t n = 0, N = instancePoints.size(); n < N; ++n) {
         if (instanceRadius[n] > largestRadius) {
             largestRadius = instanceRadius[n];
-            largestRadiusIdx = n;
+            largestRadiusIdx = int(n);
         }
     }
 
     Vec3s pos;
     Vec4s sphere;
-    minRadius *= transform.voxelSize()[0];
-    maxRadius *= transform.voxelSize()[0];
+    minRadius = float(minRadius * transform.voxelSize()[0]);
+    maxRadius = float(maxRadius * transform.voxelSize()[0]);
 
     for (size_t s = 0, S = std::min(size_t(maxSphereCount), instancePoints.size()); s < S; ++s) {
 
@@ -822,7 +821,8 @@ fillWithSpheres(
         spheres.push_back(sphere);
         instanceMask[largestRadiusIdx] = 1;
 
-        internal::UpdatePoints op(sphere, instancePoints, instanceRadius, instanceMask, overlapping);
+        internal::UpdatePoints op(
+            sphere, instancePoints, instanceRadius, instanceMask, overlapping);
         op.run();
 
         largestRadius = op.radius();
@@ -863,9 +863,9 @@ ClosestSurfacePoint<GridT>::initialize(
     const GridT& grid, float isovalue, InterrupterT* interrupter)
 {
     mIsInitialized = false;
-    typedef tree::LeafManager<const TreeT> LeafManagerT;
-    typedef tree::LeafManager<IntTreeT>    IntLeafManagerT;
-    typedef tree::LeafManager<Int16TreeT>  Int16LeafManagerT;
+    typedef tree::LeafManager<const TreeT>      LeafManagerT;
+    typedef tree::LeafManager<Index32TreeT>     Index32LeafManagerT;
+    typedef tree::LeafManager<Int16TreeT>       Int16LeafManagerT;
     typedef typename GridT::ValueType ValueT;
 
     const TreeT& tree = grid.tree();
@@ -873,41 +873,50 @@ ClosestSurfacePoint<GridT>::initialize(
 
     { // Extract surface point cloud
 
-        {
-            LeafManagerT leafs(tree);
-            internal::SignData<TreeT, LeafManagerT>
-                signDataOp(tree, leafs, ValueT(isovalue));
+        mSignTreePt.reset(new Int16TreeT(0));
+        mIdxTreePt.reset(new Index32TreeT(boost::integer_traits<Index32>::const_max));
 
-            signDataOp.run();
-
-            mSignTreePt = signDataOp.signTree();
-            mIdxTreePt = signDataOp.idxTree();
-        }
+        BoolTreeT mask(false);
+        volume_to_mesh_internal::identifySurfaceIntersectingVoxels(mask, tree, ValueT(isovalue));
+        volume_to_mesh_internal::computeAuxiliaryData(*mSignTreePt, *mIdxTreePt, mask, tree, ValueT(isovalue));
 
         if (interrupter && interrupter->wasInterrupted()) return;
 
-        Int16LeafManagerT signLeafs(*mSignTreePt);
+        // count unique points
 
-        std::vector<size_t> regions(signLeafs.leafCount(), 0);
-        signLeafs.foreach(internal::CountPoints(regions));
+        typedef typename Int16TreeT::LeafNodeType   Int16LeafNodeType;
+        typedef typename Index32TreeT::LeafNodeType Index32LeafNodeType;
 
-        mPointListSize = 0;
-        for (size_t tmp = 0, n = 0, N = regions.size(); n < N; ++n) {
-            tmp = regions[n];
-            regions[n] = mPointListSize;
-            mPointListSize += tmp;
+        std::vector<Int16LeafNodeType*> signFlagsLeafNodes;
+        mSignTreePt->getNodes(signFlagsLeafNodes);
+
+        const tbb::blocked_range<size_t> auxiliaryLeafNodeRange(0, signFlagsLeafNodes.size());
+
+        boost::scoped_array<Index32> leafNodeOffsets(new Index32[signFlagsLeafNodes.size()]);
+
+        tbb::parallel_for(auxiliaryLeafNodeRange,
+            volume_to_mesh_internal::LeafNodePointCount<Int16LeafNodeType::LOG2DIM>
+                (signFlagsLeafNodes, leafNodeOffsets));
+
+        {
+            Index32 pointCount = 0;
+            for (size_t n = 0, N = signFlagsLeafNodes.size(); n < N; ++n) {
+                const Index32 tmp = leafNodeOffsets[n];
+                leafNodeOffsets[n] = pointCount;
+                pointCount += tmp;
+            }
+
+            mPointListSize = size_t(pointCount);
+            mSurfacePointList.reset(new Vec3s[mPointListSize]);
         }
 
-        if (mPointListSize == 0) return;
 
-        mSurfacePointList.reset(new Vec3s[mPointListSize]);
+        std::vector<Index32LeafNodeType*> pointIndexLeafNodes;
+        mIdxTreePt->getNodes(pointIndexLeafNodes);
 
-        internal::GenPoints<TreeT, Int16LeafManagerT>
-            pointOp(signLeafs, tree, *mIdxTreePt, mSurfacePointList, regions, transform, isovalue);
-
-        pointOp.run();
-
-        mIdxTreePt->topologyUnion(*mSignTreePt);
+        tbb::parallel_for(auxiliaryLeafNodeRange,
+            volume_to_mesh_internal::ComputePoints<TreeT>(mSurfacePointList.get(), tree, pointIndexLeafNodes,
+                signFlagsLeafNodes, leafNodeOffsets, transform, ValueT(isovalue)));
     }
 
     if (interrupter && interrupter->wasInterrupted()) return;
@@ -923,26 +932,24 @@ ClosestSurfacePoint<GridT>::initialize(
     dim[2] = std::abs(dim[2]);
 
     mMaxRadiusSqr = std::min(std::min(dim[0], dim[1]), dim[2]);
-    mMaxRadiusSqr *= 0.51;
+    mMaxRadiusSqr *= 0.51f;
     mMaxRadiusSqr *= mMaxRadiusSqr;
 
 
-    IntLeafManagerT idxLeafs(*mIdxTreePt);
+    Index32LeafManagerT idxLeafs(*mIdxTreePt);
 
+    typedef typename Index32TreeT::RootNodeType Index32RootNodeT;
+    typedef typename Index32RootNodeT::NodeChainType Index32NodeChainT;
+    BOOST_STATIC_ASSERT(boost::mpl::size<Index32NodeChainT>::value > 1);
+    typedef typename boost::mpl::at<Index32NodeChainT, boost::mpl::int_<1> >::type Index32InternalNodeT;
 
-    typedef typename IntTreeT::RootNodeType IntRootNodeT;
-    typedef typename IntRootNodeT::NodeChainType IntNodeChainT;
-    BOOST_STATIC_ASSERT(boost::mpl::size<IntNodeChainT>::value > 1);
-    typedef typename boost::mpl::at<IntNodeChainT, boost::mpl::int_<1> >::type IntInternalNodeT;
+    typename Index32TreeT::NodeCIter nIt = mIdxTreePt->cbeginNode();
+    nIt.setMinDepth(Index32TreeT::NodeCIter::LEAF_DEPTH - 1);
+    nIt.setMaxDepth(Index32TreeT::NodeCIter::LEAF_DEPTH - 1);
 
+    std::vector<const Index32InternalNodeT*> internalNodes;
 
-    typename IntTreeT::NodeCIter nIt = mIdxTreePt->cbeginNode();
-    nIt.setMinDepth(IntTreeT::NodeCIter::LEAF_DEPTH - 1);
-    nIt.setMaxDepth(IntTreeT::NodeCIter::LEAF_DEPTH - 1);
-
-    std::vector<const IntInternalNodeT*> internalNodes;
-
-    const IntInternalNodeT* node = NULL;
+    const Index32InternalNodeT* node = NULL;
     for (; nIt; ++nIt) {
         nIt.getNode(node);
         if (node) internalNodes.push_back(node);
@@ -951,10 +958,10 @@ ClosestSurfacePoint<GridT>::initialize(
     std::vector<IndexRange>().swap(mLeafRanges);
     mLeafRanges.resize(internalNodes.size());
 
-    std::vector<const IntLeafT*>().swap(mLeafNodes);
+    std::vector<const Index32LeafT*>().swap(mLeafNodes);
     mLeafNodes.reserve(idxLeafs.leafCount());
 
-    typename IntInternalNodeT::ChildOnCIter leafIt;
+    typename Index32InternalNodeT::ChildOnCIter leafIt;
     mMaxNodeLeafs = 0;
     for (size_t n = 0, N = internalNodes.size(); n < N; ++n) {
 
@@ -974,7 +981,8 @@ ClosestSurfacePoint<GridT>::initialize(
     std::vector<Vec4R>().swap(mLeafBoundingSpheres);
     mLeafBoundingSpheres.resize(mLeafNodes.size());
 
-    internal::LeafBS<IntLeafT> leafBS(mLeafBoundingSpheres, mLeafNodes, transform, mSurfacePointList);
+    internal::LeafBS<Index32LeafT> leafBS(
+        mLeafBoundingSpheres, mLeafNodes, transform, mSurfacePointList);
     leafBS.run();
 
 
@@ -997,7 +1005,7 @@ ClosestSurfacePoint<GridT>::search(std::vector<Vec3R>& points,
     distances.clear();
     distances.resize(points.size(), mMaxRadiusSqr);
 
-    internal::ClosestPointDist<IntLeafT> cpd(points, distances, mSurfacePointList,
+    internal::ClosestPointDist<Index32LeafT> cpd(points, distances, mSurfacePointList,
         mLeafNodes, mLeafRanges, mLeafBoundingSpheres, mNodeBoundingSpheres,
         mMaxNodeLeafs, transformPoints);
 
@@ -1017,7 +1025,8 @@ ClosestSurfacePoint<GridT>::search(const std::vector<Vec3R>& points, std::vector
 
 template<typename GridT>
 bool
-ClosestSurfacePoint<GridT>::searchAndReplace(std::vector<Vec3R>& points, std::vector<float>& distances)
+ClosestSurfacePoint<GridT>::searchAndReplace(std::vector<Vec3R>& points,
+    std::vector<float>& distances)
 {
     return search(points, distances, true);
 }
@@ -1029,6 +1038,6 @@ ClosestSurfacePoint<GridT>::searchAndReplace(std::vector<Vec3R>& points, std::ve
 
 #endif // OPENVDB_TOOLS_VOLUME_TO_MESH_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2013 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
