@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -46,17 +46,19 @@ public:
         Float5432Tree;
     typedef openvdb::Grid<Float5432Tree> Float5432Grid;
 
-    virtual void setUp() { openvdb::initialize(); }
+    virtual void setUp()    { openvdb::initialize(); }
     virtual void tearDown() { openvdb::uninitialize(); }
 
     CPPUNIT_TEST_SUITE(TestGridIO);
     CPPUNIT_TEST(testReadAllBool);
+    CPPUNIT_TEST(testReadAllMask);
     CPPUNIT_TEST(testReadAllFloat);
     CPPUNIT_TEST(testReadAllVec3S);
     CPPUNIT_TEST(testReadAllFloat5432);
     CPPUNIT_TEST_SUITE_END();
 
-    void testReadAllBool() { readAllTest<openvdb::BoolGrid>(); }
+    void testReadAllBool()  { readAllTest<openvdb::BoolGrid>(); }
+    void testReadAllMask()  { readAllTest<openvdb::MaskGrid>(); }
     void testReadAllFloat() { readAllTest<openvdb::FloatGrid>(); }
     void testReadAllVec3S() { readAllTest<openvdb::Vec3SGrid>(); }
     void testReadAllFloat5432() { Float5432Grid::registerGrid(); readAllTest<Float5432Grid>(); }
@@ -229,6 +231,6 @@ TestGridIO::readAllTest()
     ::remove("something.vdb2");
 }
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
