@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -223,47 +223,61 @@ TestLeafBool::testEquivalence()
 {
     using openvdb::CoordBBox;
     using openvdb::Coord;
-
-    LeafType leaf(Coord(0, 0, 0), false); // false and inactive
-    LeafType leaf2(Coord(0, 0, 0), true); // true and inactive
-
-    CPPUNIT_ASSERT(leaf != leaf2);
-
-    leaf.fill(CoordBBox(Coord(0), Coord(LeafType::DIM - 1)), true, /*active=*/false);
-    CPPUNIT_ASSERT(leaf == leaf2); // true and inactive
-
-    leaf.setValuesOn(); // true and active
-
-    leaf2.fill(CoordBBox(Coord(0), Coord(LeafType::DIM - 1)), false); // false and active
-    CPPUNIT_ASSERT(leaf != leaf2);
-
-    leaf.negate(); // false and active
-    CPPUNIT_ASSERT(leaf == leaf2);
-
-    // Set some values.
-    leaf.setValueOn(Coord(0, 0, 0), true);
-    leaf.setValueOn(Coord(0, 1, 0), true);
-    leaf.setValueOn(Coord(1, 1, 0), true);
-    leaf.setValueOn(Coord(1, 1, 2), true);
-
-    leaf2.setValueOn(Coord(0, 0, 0), true);
-    leaf2.setValueOn(Coord(0, 1, 0), true);
-    leaf2.setValueOn(Coord(1, 1, 0), true);
-    leaf2.setValueOn(Coord(1, 1, 2), true);
-
-    CPPUNIT_ASSERT(leaf == leaf2);
-
-    leaf2.setValueOn(Coord(0, 0, 1), true);
-
-    CPPUNIT_ASSERT(leaf != leaf2);
-
-    leaf2.setValueOff(Coord(0, 0, 1), false);
-
-    CPPUNIT_ASSERT(leaf != leaf2);
-
-    leaf2.setValueOn(Coord(0, 0, 1));
-
-    CPPUNIT_ASSERT(leaf == leaf2);
+    {
+        LeafType leaf(Coord(0, 0, 0), false); // false and inactive
+        LeafType leaf2(Coord(0, 0, 0), true); // true and inactive
+        
+        CPPUNIT_ASSERT(leaf != leaf2);
+        
+        leaf.fill(CoordBBox(Coord(0), Coord(LeafType::DIM - 1)), true, /*active=*/false);
+        CPPUNIT_ASSERT(leaf == leaf2); // true and inactive
+        
+        leaf.setValuesOn(); // true and active
+        
+        leaf2.fill(CoordBBox(Coord(0), Coord(LeafType::DIM - 1)), false); // false and active
+        CPPUNIT_ASSERT(leaf != leaf2);
+        
+        leaf.negate(); // false and active
+        CPPUNIT_ASSERT(leaf == leaf2);
+        
+        // Set some values.
+        leaf.setValueOn(Coord(0, 0, 0), true);
+        leaf.setValueOn(Coord(0, 1, 0), true);
+        leaf.setValueOn(Coord(1, 1, 0), true);
+        leaf.setValueOn(Coord(1, 1, 2), true);
+        
+        leaf2.setValueOn(Coord(0, 0, 0), true);
+        leaf2.setValueOn(Coord(0, 1, 0), true);
+        leaf2.setValueOn(Coord(1, 1, 0), true);
+        leaf2.setValueOn(Coord(1, 1, 2), true);
+        
+        CPPUNIT_ASSERT(leaf == leaf2);
+        
+        leaf2.setValueOn(Coord(0, 0, 1), true);
+        
+        CPPUNIT_ASSERT(leaf != leaf2);
+        
+        leaf2.setValueOff(Coord(0, 0, 1), false);
+        
+        CPPUNIT_ASSERT(leaf != leaf2);
+        
+        leaf2.setValueOn(Coord(0, 0, 1));
+        
+        CPPUNIT_ASSERT(leaf == leaf2);
+    }
+    {// test LeafNode<bool>::operator==()
+        LeafType leaf1(Coord(0            , 0, 0), true); // true and inactive
+        LeafType leaf2(Coord(1            , 0, 0), true); // true and inactive
+        LeafType leaf3(Coord(LeafType::DIM, 0, 0), true); // true and inactive
+        LeafType leaf4(Coord(0            , 0, 0), true, true);//true and active
+        CPPUNIT_ASSERT(leaf1 == leaf2);
+        CPPUNIT_ASSERT(leaf1 != leaf3);
+        CPPUNIT_ASSERT(leaf2 != leaf3);
+        CPPUNIT_ASSERT(leaf1 != leaf4);
+        CPPUNIT_ASSERT(leaf2 != leaf4);
+        CPPUNIT_ASSERT(leaf3 != leaf4);
+    }
+        
 }
 
 
@@ -558,6 +572,6 @@ TestLeafBool::testBoolTree()
 //     CPPUNIT_ASSERT(tree->hasSameTopology(*copyOfTree));
 // }
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
