@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -44,11 +44,13 @@ public:
     CPPUNIT_TEST(testSDFToFogVolume);
     CPPUNIT_TEST(testSDFInteriorMask);
     CPPUNIT_TEST(testExtractEnclosedRegion);
+    CPPUNIT_TEST(testSegmentationTools);
     CPPUNIT_TEST_SUITE_END();
 
     void testSDFToFogVolume();
     void testSDFInteriorMask();
     void testExtractEnclosedRegion();
+    void testSegmentationTools();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestLevelSetUtil);
@@ -113,8 +115,8 @@ TestLevelSetUtil::testExtractEnclosedRegion()
     typedef openvdb::math::BBox<Vec3s>  BBoxs;
     typedef openvdb::math::Transform    Transform;
 
-    BBoxs regionA(Vec3s(0.0, 0.0, 0.0), Vec3s(3.0, 3.0, 3.0));
-    BBoxs regionB(Vec3s(1.0, 1.0, 1.0), Vec3s(2.0, 2.0, 2.0));
+    BBoxs regionA(Vec3s(0.0f, 0.0f, 0.0f), Vec3s(3.0f, 3.0f, 3.0f));
+    BBoxs regionB(Vec3s(1.0f, 1.0f, 1.0f), Vec3s(2.0f, 2.0f, 2.0f));
 
     Transform::Ptr transform = Transform::createLinearTransform(0.1);
 
@@ -135,6 +137,65 @@ TestLevelSetUtil::testExtractEnclosedRegion()
 }
 
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+void
+TestLevelSetUtil::testSegmentationTools()
+{
+    typedef openvdb::FloatGrid          FloatGrid;
+    typedef openvdb::Vec3s              Vec3s;
+    typedef openvdb::math::BBox<Vec3s>  BBoxs;
+    typedef openvdb::math::Transform    Transform;
+
+    { // Test SDF segmentation
+
+        // Create two sdf boxes with overlapping narrow-bands.
+        BBoxs regionA(Vec3s(0.0f, 0.0f, 0.0f), Vec3s(2.0f, 2.0f, 2.0f));
+        BBoxs regionB(Vec3s(2.5f, 0.0f, 0.0f), Vec3s(4.3f, 2.0f, 2.0f));
+
+        Transform::Ptr transform = Transform::createLinearTransform(0.1);
+
+        FloatGrid::Ptr sdfGrid = openvdb::tools::createLevelSetBox<FloatGrid>(regionA, *transform);
+        FloatGrid::Ptr sdfGridB = openvdb::tools::createLevelSetBox<FloatGrid>(regionB, *transform);
+
+        openvdb::tools::csgUnion(*sdfGrid, *sdfGridB);
+
+        std::vector<FloatGrid::Ptr> segments;
+
+        // This tool will not identify two separate segments when the narrow-bands overlap. 
+        openvdb::tools::segmentActiveVoxels(*sdfGrid, segments);
+        CPPUNIT_ASSERT(segments.size() == 1);
+
+        segments.clear();
+
+        // This tool should properly identify two separate segments
+        openvdb::tools::segmentSDF(*sdfGrid, segments);
+        CPPUNIT_ASSERT(segments.size() == 2);
+
+
+        // test inside ls region coord value
+        openvdb::Coord ijk = transform->worldToIndexNodeCentered(openvdb::Vec3d(1.5, 1.5, 1.5));
+        CPPUNIT_ASSERT(segments[0]->tree().getValue(ijk) < 0.0f);
+
+        // test outside coord value
+        ijk = transform->worldToIndexNodeCentered(openvdb::Vec3d(3.5, 3.5, 3.5));
+        CPPUNIT_ASSERT(segments[0]->tree().getValue(ijk) > 0.0f);
+    }
+
+    { // Test fog volume with active tiles
+
+        openvdb::FloatGrid::Ptr grid = openvdb::FloatGrid::create(0.0);
+
+        grid->fill(openvdb::CoordBBox(openvdb::Coord(0), openvdb::Coord(50)), 1.0);
+        grid->fill(openvdb::CoordBBox(openvdb::Coord(60), openvdb::Coord(100)), 1.0);
+
+        CPPUNIT_ASSERT(grid->tree().hasActiveTiles() == true);
+
+        std::vector<FloatGrid::Ptr> segments;
+        openvdb::tools::segmentActiveVoxels(*grid, segments);
+        CPPUNIT_ASSERT(segments.size() == 2);
+    }
+}
+
+
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
