@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -32,6 +32,7 @@
 #include <openvdb/Exceptions.h>
 #include <openvdb/openvdb.h>
 #include <openvdb/tools/Interpolation.h>
+#include <openvdb/math/Stencils.h>
 
 // CPPUNIT_TEST_SUITE() invokes CPPUNIT_TESTNAMER_DECL() to generate a suite name
 // from the FixtureType.  But if FixtureType is a templated type, the generated name
@@ -66,6 +67,7 @@ public:
     CPPUNIT_TEST(testConstantValues);
     CPPUNIT_TEST(testFillValues);
     CPPUNIT_TEST(testNegativeIndices);
+    CPPUNIT_TEST(testStencilsMatch);
     CPPUNIT_TEST_SUITE_END();
 
     void test();
@@ -74,6 +76,7 @@ public:
     void testConstantValues();
     void testFillValues();
     void testNegativeIndices();
+    void testStencilsMatch();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestLinearInterp<openvdb::FloatGrid>);
@@ -123,97 +126,97 @@ TestLinearInterp<GridType>::test()
     tree.setValue(openvdb::Coord(11,  9, 9), 4.0);
 
     {//using BoxSampler
-    
+
         // transform used for worldspace interpolation)
         openvdb::tools::GridSampler<GridType, openvdb::tools::BoxSampler>
             interpolator(grid);
         //openvdb::tools::LinearInterp<GridType> interpolator(*tree);
-        
+
         typename GridType::ValueType val =
             interpolator.sampleVoxel(10.5, 10.5, 10.5);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.375, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.0, 10.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(11.0, 10.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(11.0, 11.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(11.0, 11.0, 11.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(9.0, 11.0, 9.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(9.0, 10.0, 9.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.1, 10.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.1, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.8, 10.8, 10.8);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.792, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.1, 10.8, 10.5);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.41, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.8, 10.1, 10.5);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.41, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.5, 10.1, 10.8);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.71, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.5, 10.8, 10.1);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.01, val, TOLERANCE);
 
     }
     {//using Sampler<1>
-        
+
         // transform used for worldspace interpolation)
-        openvdb::tools::GridSampler<GridType, openvdb::tools::Sampler<1> > 
+        openvdb::tools::GridSampler<GridType, openvdb::tools::Sampler<1> >
             interpolator(grid);
         //openvdb::tools::LinearInterp<GridType> interpolator(*tree);
-        
+
         typename GridType::ValueType val =
             interpolator.sampleVoxel(10.5, 10.5, 10.5);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.375, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.0, 10.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(11.0, 10.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(11.0, 11.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(11.0, 11.0, 11.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(9.0, 11.0, 9.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(9.0, 10.0, 9.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.1, 10.0, 10.0);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.1, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.8, 10.8, 10.8);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.792, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.1, 10.8, 10.5);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.41, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.8, 10.1, 10.5);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.41, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.5, 10.1, 10.8);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.71, val, TOLERANCE);
-        
+
         val = interpolator.sampleVoxel(10.5, 10.8, 10.1);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(2.01, val, TOLERANCE);
     }
@@ -1049,6 +1052,50 @@ TestLinearInterp<openvdb::Vec3SGrid>::testNegativeIndices()
     CPPUNIT_ASSERT(val.eq(Vec3s(2.01f)));
 }
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+
+template<typename GridType>
+void
+TestLinearInterp<GridType>::testStencilsMatch()
+{
+    typedef typename GridType::ValueType ValueType;
+
+    GridType grid;
+    typename GridType::TreeType& tree = grid.tree();
+
+    // using mostly recurring numbers
+
+    tree.setValue(openvdb::Coord(0, 0, 0), ValueType(1.0/3.0));
+    tree.setValue(openvdb::Coord(0, 1, 0), ValueType(1.0/11.0));
+    tree.setValue(openvdb::Coord(0, 0, 1), ValueType(1.0/81.0));
+    tree.setValue(openvdb::Coord(1, 0, 0), ValueType(1.0/97.0));
+    tree.setValue(openvdb::Coord(1, 1, 0), ValueType(1.0/61.0));
+    tree.setValue(openvdb::Coord(0, 1, 1), ValueType(9.0/7.0));
+    tree.setValue(openvdb::Coord(1, 0, 1), ValueType(9.0/11.0));
+    tree.setValue(openvdb::Coord(1, 1, 1), ValueType(22.0/7.0));
+
+    const openvdb::Vec3f pos(7.0f/12.0f, 1.0f/3.0f, 2.0f/3.0f);
+
+    {//using BoxSampler and BoxStencil
+
+        openvdb::tools::GridSampler<GridType, openvdb::tools::BoxSampler>
+            interpolator(grid);
+
+        openvdb::math::BoxStencil<const GridType>
+            stencil(grid);
+
+        typename GridType::ValueType val1 = interpolator.sampleVoxel(pos.x(), pos.y(), pos.z());
+
+        stencil.moveTo(pos);
+        typename GridType::ValueType val2 = stencil.interpolation(pos);
+        CPPUNIT_ASSERT_EQUAL(val1, val2);
+    }
+}
+
+template<>
+void
+TestLinearInterp<openvdb::Vec3SGrid>::testStencilsMatch() {}
+
+
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
