@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -89,7 +89,7 @@ TestMaps::testApproxInverse()
         Mat4d singularInv = approxInverse(mat);
 
         // this matrix is equal to its own singular inverse
-        CPPUNIT_ASSERT( mat == singularInv );
+        CPPUNIT_ASSERT( mat.eq(singularInv) );
 
     }
     {
@@ -829,6 +829,6 @@ TestMaps::testJacobians()
 
 
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
