@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -42,6 +42,7 @@ class TestStats: public CppUnit::TestCase
 {
 public:
     CPPUNIT_TEST_SUITE(TestStats);
+    CPPUNIT_TEST(testMinMax);
     CPPUNIT_TEST(testExtrema);
     CPPUNIT_TEST(testStats);
     CPPUNIT_TEST(testHistogram);
@@ -51,6 +52,7 @@ public:
     CPPUNIT_TEST(testGridOperatorStats);
     CPPUNIT_TEST_SUITE_END();
 
+    void testMinMax();
     void testExtrema();
     void testStats();
     void testHistogram();
@@ -62,6 +64,25 @@ public:
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestStats);
 
+void
+TestStats::testMinMax()
+{
+    // test Coord which uses lexicographic less than
+    openvdb::math::MinMax<openvdb::Coord> s(openvdb::Coord::max(), openvdb::Coord::min());
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord::max(), s.min());
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord::min(), s.max());
+    s.add( openvdb::Coord(1,2,3) );
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord(1,2,3), s.min());
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord(1,2,3), s.max());
+    s.add( openvdb::Coord(0,2,3) );
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord(0,2,3), s.min());
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord(1,2,3), s.max());
+    s.add( openvdb::Coord(1,2,4) );
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord(0,2,3), s.min());
+    CPPUNIT_ASSERT_EQUAL(openvdb::Coord(1,2,4), s.max());
+}
+
+
 void
 TestStats::testExtrema()
 {
@@ -717,6 +738,6 @@ TestStats::testGridHistogram()
     }
 }
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2016 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
